% ----------------------------------------------------
% - VMCGen - Vertical Motion Canceller GENerator v1.0
% copyright(c)2001-05 RAGUODENKO,all rights reserved.
% this program can be redistributed or modified under
% terms of Berkeley-based copyrights,see LICENSE.TXT.
% ----------------------------------------------------
%
% Usage:
%  F=VMCGen(B,D)
%   B: location of Joint B of follower linkage (nx2 matrix).
%   D: location of Joint D of driver linkage, assuming (cos(phi),sin(phi)+Ay).
%   F: trace of Curve-F. ((n-1)x2 matrix).

function F=VMCGen(B,D)

	nB=size(B); n=nB(1);
	Bx=B(:,1); By=B(:,2);
	Dx=D(:,1); Dy=D(:,2);

	% FIND THE POINT-S, HERE S-LINE IS:
	% (Dy-By)x-(Dx-Bx)y+((Dx-Bx)Dy-(Dy-By)Dx)=0

	 Sx=-((Dx-Bx).*Dy-(Dy-By).*Dx)./(Dy-By);
	 Sy=zeros(n,1);

	% FIND THE POINT-P, HERE P-LINE IS:
	% (Dx-Bx)x+(Dy-By)y+((Dy-By)Dy+(Dx-Bx)Dx)=0

	 Px=((Dy-By).*Dy+(Dx-Bx).*Dx)./(Dx-Bx);
	 Py=zeros(n,1);

	% TRANSLATION xy-AXIS XY-AXIS IS:
	% SXY=E(Sxy-D), PXY=E(Pxy-D), HERE;
	% E=1/norm(D-B)*[-(Dy-By)   Dx-Bx
	%                -(Dx-Bx) -(Dy-By)]

	 det=sqrt((Dx-Bx).^2+(Dy-By).^2);
	 Ls=(1./det).*(-(Dx-Bx).*(Sx-Dx)-(Dy-By).*(Sy-Dy));
	 Lp=(1./det).*(-(Dy-By).*(Px-Dx)+(Dx-Bx).*(Py-Dy));

	% SOLVE THE SIMULTANIOUS EQUATION:
	%  Ls(p   )X+Lp(p   )Y=Ls(p   )Lp(p   )
	%  Ls(p+dp)X+Lp(p+dp)Y=Ls(p+dp)Lp(p+dp)

	 k=1:(n-1);
	 Det=Ls(k).*Lp(k+1)-Lp(k).*Ls(k+1);
 
	 Fx=(1./Det).*( Lp(k+1).*Lp(k).*(Ls(k  )-Ls(k+1)));
	 Fy=(1./Det).*( Ls(k+1).*Ls(k).*(Lp(k+1)-Lp(k  )));
	 F=[Fx Fy];
end


